#+TITLE: Using and defining modules

This chapter describes how to extend GNU Emacs dynamically with
natively compiled code, as opposed to loading Emacs Lisp or
byte-compiled Emacs Lisp code.

* Usage

Starting with version 25, GNU Emacs includes support for loading
dynamic modules. This feature is still considered experimental and is
not enabled by default. You need to build Emacs with the
=--with-modules= configure option to enable it.

You can check your build of Emacs supports modules with =(fbound 'module-load)=.
It should return =t= when supported, =nil= otherwise.

When enabled, Emacs can load your platform native shared libraries
file format. These files typically ends with one of the following
extensions: =.dll=, =.so=, =.dylib=, ... The actual suffix that Emacs
uses to look for those files on your system is set in the
=module-file-suffix= variable.

When loading a package via =load= or =require=, Emacs will also look
for modules, using the module suffix. See [[Loading]].

* Defining a dynamic module

Modules work as "plugins" in many other programs. Each module is
compiled to a shared library. This shared library implements functions
that will get called by the Emacs process. The plugin API or module
interface describes what Emacs expects as entry points and what the
module can use to interact with Emacs.

Concretely this API is accessible through the =emacs-module.h= header
file. This is the only thing you need to make an Emacs module, you
don't have to link against any Emacs binaries or libraries.

The most important things this file defines is:

- An =emacs_value= opaque type which is used to pass Lisp values to
  and from Emacs.
- An =emacs_runtime= structure which is used to get an environment pointer.
- An =emacs_env= structure that contains function pointers to the API.
- The function prototype of the module main entry point.

#+BEGIN_SRC c
extern int emacs_module_init (struct emacs_runtime *ert);
#+END_SRC

This is the function Emacs will call when it loads your module. It
will pass you a pointer to a runtime structure, from which you can get
an environement pointer:

#+BEGIN_SRC
emacs_env *env = ert->get_environment(ert);
#+END_SRC

This environement pointer lets you interact with the Lisp interpreter
through function pointers. All those function pointers take the
environment as the first parameter.

* Basic module

We're going to define a new module =mymod= that defines a function
=mymod-test= that returns the integer 42.

The only requirement is a copy of the =emacs-module.h= file. You can
copy it from the Emacs sources or point your compiler to Emacs sources
directory. We will do the later for this example.

The following Makefile assumes a POSIX system with the =gcc= compiler
available. Adapt it according to your platform.

** Makefile

#+BEGIN_SRC makefile
# path to the emacs source dir
# (you can provide it here or on the command line)
#ROOT    =
CC      = gcc
LD      = gcc
CFLAGS  = -ggdb3 -Wall
LDFLAGS =

all: mymod.so

# make shared library out of the object file
%.so: %.o
        $(LD) -shared $(LDFLAGS) -o $@ $<

# compile source file to object file
%.o: %.c
        $(CC) $(CFLAGS) -I$(ROOT)/src -fPIC -c $<

#+END_SRC

** =mymod.c=

#+BEGIN_SRC c
#include <emacs-module.h>
#+END_SRC

To use the module API you need the =emacs-module.h= header. If you're
using the makefile above, you can see the =-I= flag that tells =gcc=
to look in Emacs sources. The API is designed so that you don't need
to link against any Emacs binaries. All of the API is made available
through function pointers given to your module when it is loaded.

#+BEGIN_SRC c
/* Declare mandatory GPL symbol.  */
int plugin_is_GPL_compatible;
#+END_SRC

Emacs modules must be GPL compatible. To enforce that Emacs checks for
the presence of the =plugin_is_GPL_compatible= symbol before it loads
a module.

#+BEGIN_SRC c
/* New emacs lisp function. All function exposed to Emacs must have this prototype. */
static emacs_value
Fmymod_test (emacs_env *env, ptrdiff_t nargs, emacs_value args[], void *data)
{
  return env->make_integer (env, 42);
}
#+END_SRC

This is how you define a C function that can be called by Emacs. They
all need to have this prototype. It doesn't have to be =static= but
it's usually a Good Thing to only expose the symbols you need to.

- =emacs_env *env= --- pointer that lets you interact with Emacs (contains all of the function pointers).
- =ptrdiff_t nargs= --- number of arguments (number of elements in =args=)
- =emacs_value args[]= --- array of arguments. You can see that Lisp
  object are represented by the opaque =emacs_value= type.
- =void *data= --- user-pointer of your choice (set when you register the function)
- The function must return an =emacs_value=.

This alone is not enough: you need to register this function so that
Emacs knows about it. This is done with =env->make_function()=. It
will wrap your C function in a Lisp object (=emacs_value= in the
module API) that Emacs can handle and call, like a =lambda= (anonymous
function).

This is how you would register =Fmymod_test=:

#+BEGIN_SRC c
/* create a lambda (returns an emacs_value) */
emacs_value fun = env->make_function (env,
            0,            /* min. number of arguments */
            0,            /* max. number of arguments */
            Fmymod_test,  /* actual function pointer */
            "doc",        /* docstring */
            NULL          /* user pointer of your choice (data param in Fmymod_test) */
);
#+END_SRC

Let's define a helper function that binds a function value to a
symbol. Since Emacs Lisp is a /Lisp-2/, you need to set the symbol
function-cell to your function value. In Emacs Lisp that would be done
with =(fset 'mymod-test (lamba () 42))=. Let's make that exact call:

#+BEGIN_SRC c
/* Bind NAME to FUN.  */
static void
bind_function (emacs_env *env, const char *name, emacs_value Sfun)
{
  /* Set the function cell of the symbol named NAME to SFUN using
     the 'fset' function.  */

  /* Convert the strings to symbols by interning them */
  emacs_value Qfset = env->intern (env, "fset");
  emacs_value Qsym = env->intern (env, name);

  /* Prepare the arguments array */
  emacs_value args[] = { Qsym, Sfun };

  /* Make the call (2 == nb of arguments) */
  env->funcall (env, Qfset, 2, args);
}
#+END_SRC

Since we cannot rely on C type system to represent Lisp values, it's
good practice to use a notation to identify the type of the thing
stored in an =emacs_value=. Here we use the =Q= prefix for symbols (/Q/
as in /quote/) and =S= for /Subroutines/.

Now let's make a helper function that register the current module as
loaded (like you would in an Emacs Lisp package) by calling =provide=.

#+BEGIN_SRC c
/* Provide FEATURE to Emacs.  */
static void
provide (emacs_env *env, const char *feature)
{
  /* call 'provide' with FEATURE converted to a symbol */

  emacs_value Qfeat = env->intern (env, feature);
  emacs_value Qprovide = env->intern (env, "provide");
  emacs_value args[] = { Qfeat };

  env->funcall (env, Qprovide, 1, args);
}
#+END_SRC

Finally, let's define the module entry point: the C function Emacs
will call when you load the module. It has to be called =emacs_module_init=.

#+BEGIN_SRC c
int
emacs_module_init (struct emacs_runtime *ert)
{
  emacs_env *env = ert->get_environment (ert);

  /* create a lambda (returns an emacs_value) */
  emacs_value fun = env->make_function (env,
              0,            /* min. number of arguments */
              0,            /* max. number of arguments */
              Fmymod_test,  /* actual function pointer */
              "doc",        /* docstring */
              NULL          /* user pointer of your choice (data param in Fmymod_test) */
  );

  bind_function (env, "mymod-test", fun);
  provide (env, "mymod");

  /* loaded successfully */
  return 0;
#+END_SRC

** Testing

Compile your module by providing the path to the Emacs source
directory as the =ROOT= variable. For that you can edit the =Makefile=
or provide it via the command line like so:

#+BEGIN_EXAMPLE
$ make ROOT=$HOME/prog/emacs-25.1
gcc -ggdb3 -Wall -I/home/aaptel/prog/emacs-25.1/src -fPIC -c mymod.c
gcc -shared  -o mymod.so mymod.o
rm mymod.o
#+END_EXAMPLE

You can now run your modules-enabled Emacs, load and test your
module. Use =-L= to append the directory containing your compiled
module to the load-path list so that Emacs can find it.

#+BEGIN_EXAMPLE
$ ~/prog/emacs-25.1/src/emacs -Q -L $PWD
#+END_EXAMPLE

Load your module and try to call =mymod-test=. If you have done
everything right you should get this:

#+BEGIN_EXAMPLE
(require 'mymod)
=> mymod

(mymod-test)
=> 42
#+END_EXAMPLE


* API Referrence
** Environment functions

These functions pointers are in the =emacs_env= structure.

*** Memory management

When a module function gets called by Emacs, all the parameters given
(environment and arguments) and all =emacs_value= allocated within
the function *are only valid until the module function returns*. You
cannot store these for later use or between module function
calls. Modules cannot interact with the module API outside of module
function calls and the module entry point.

XXX: the global ref mechanism doesn't make sense... How can you get a
GC protected value back?  Since the emacs_value is only valid in the
module function lifetime, a user would need to store it or directly
pass it in another call in which case the value would already be
considered "in-use" by the GC anyway. Making the GC protection
useless. I actually cannot think of a usecase for this. I believe we
introduced it in a previous design where we didnt invalidate
emacs_values between calls, hence we could keep them, making them
possible targets for the GC.


#+BEGIN_SRC
emacs_value (*make_global_ref) (emacs_env *env, emacs_value any_reference);
#+END_SRC

Prevent =any_reference= from being garbage-collected when it's not
referenced outside of your function.

#+BEGIN_SRC c
void (*free_global_ref) (emacs_env *env, emacs_value global_reference);
#+END_SRC

Remove the GC protection previously set with =make_global_ref()=.


*** Type conversion

Integer conversion uses your platform =intmax_t= type.

#+BEGIN_SRC c
intmax_t (*extract_integer) (emacs_env *env, emacs_value value);
#+END_SRC

Extract the Lisp integer stored in =value= and returns it as an
=intmax_t=. All Lisp integers can be mapped that way.

#+BEGIN_SRC c
emacs_value (*make_integer) (emacs_env *env, intmax_t value);
#+END_SRC

Convert =value= to a Lisp integer and returns it as an =emacs_value=.
Not all =intmax_t= value can be mapped that way as Emacs reserves bits
to tag values internally.

XXX: Which values?

Emacs will signal =overflow-error= when an integer cannot be mapped.

#+BEGIN_SRC c
double (*extract_float) (emacs_env *env, emacs_value value);
#+END_SRC

Extract the Lisp real number stored in =value= and returns it as a
double. All Lisp numbers can be mapped that way.

#+BEGIN_SRC c
emacs_value (*make_float) (emacs_env *env, double value);
#+END_SRC

Convert =value= to a Lisp real number and return it as an =emacs_value=.

XXX: can all values be mapped that way?

*** Making Lisp calls

Like in regular Lisp code, the intern function can be used to make a
symbol out of a string.

#+BEGIN_SRC c
  emacs_value (*intern) (emacs_env *env,
                         const char *symbol_name);
#+END_SRC

If the symbol has a value in its function cell it can be called via
=funcall=. 

#+BEGIN_SRC c
  emacs_value (*funcall) (emacs_env *env,
                          emacs_value function,
                          ptrdiff_t nargs,
                          emacs_value args[]);
#+END_SRC


*** Handling non-local exits

The Emacs signaling system (see [[Errors]]) is also available to Emacs
modules. You can:

- Check if a Lisp call ended normally (i.e. by reaching
  the end of a function) or if it ended because it signaled or threw an
  error
- Signal or throw from a module function

When a Lisp call exits 

#+BEGIN_SRC c
enum emacs_funcall_exit
{
  /* Function has returned normally.  */
  emacs_funcall_exit_return = 0,

  /* Function has signaled an error using `signal'.  */
  emacs_funcall_exit_signal = 1,

  /* Function has exit using `throw'.  */
  emacs_funcall_exit_throw = 2,
};
#+END_SRC

This enumeration defines all the possible ways a Lisp call can end.

#+BEGIN_SRC c
  enum emacs_funcall_exit (*non_local_exit_check) (emacs_env *env);
#+END_SRC

Return how the previous Lisp call ended. You have to call this
function directly after a Lisp call.

#+BEGIN_SRC c
  void (*non_local_exit_clear) (emacs_env *env);
#+END_SRC

Clear the last non-local exit status. 

#+BEGIN_SRC c
  enum emacs_funcall_exit (*non_local_exit_get)
    (emacs_env *env,
     emacs_value *non_local_exit_symbol_out,
     emacs_value *non_local_exit_data_out);
#+END_SRC

#+BEGIN_SRC c
  void (*non_local_exit_signal) (emacs_env *env,
				 emacs_value non_local_exit_symbol,
				 emacs_value non_local_exit_data);
#+END_SRC

#+BEGIN_SRC c
  void (*non_local_exit_throw) (emacs_env *env,
				emacs_value tag,
				emacs_value value);
#+END_SRC

